; Software zur Platine Pieper4 mit Bordspannungsberwachung
;**** Processor Definition
.include "tn15def.inc"
;1.6MHz System Clock

;*****************************************************************************************
;	Variablen
;*****************************************************************************************

.def		Undervoltage= r10				; undervoltage value register
.def		Temp4		= r11				; temporary register for voltage measuring
.def		AdcCount	= r12				; counter for ADC mean value calculation
.def		TimeMid		= r13				; mean value between min and max pulse time
.def		TimeL		= r14				; the sum of pulse times
.def		TimeH		= r15				; also used by adc routine after init
.def		PulseCount	= r16				; for mean value generation of pulse times
.def		Temp		= r17				; temporary register
.def		Temp2		= r18
.def		Status1 	= r19				; status register
											; bit	0	1 = pos. edge occured, wait for neg. edge
											; bits	21	00= normal operation
											; 			01= setup min. time
											;			10= setup max. time
											;			11= setup undervoltage threshold
											;		3	1 = pulse occured, for pulse missing detection
											;		4	1 =	low batt state
											;		5	1 = shit state, pulse ripple
											;		6	1 = help me state, no signal
											;		7	1 = find me state, long signal
.def		DelayTemp1	= r20				; temporary register for the different delay routines
.def		DelayTemp2	= r21
.def		DelayTemp3	= r22
.def		BeepTemp	= r23				; temporary register for beep routines
.def		BeepTemp2	= r24
.def		Temp3		= r25				; temporary register for setup routine and voltage measuring

;*****************************************************************************************
;	Werte
;*****************************************************************************************

.equ	PulseDiff	= 10					; max. time difference of subsequent pulses
											; 10 is 100s
											; pulsewidth is between 1ms and 2ms
.equ	PulseDiffSetup = 3					; max. time difference of subsequent pulses
											; during setup sequence
.equ	ThreshDiff	= 5						; difference at which the new mean value is stored to eeprom
.equ	MidDefault	= 100					; default beep threshold value if no setup is used and no value has been stored in eeprom
.equ	LowVoltage	= 4.8					; Alarm Voltage
;.equ	R1			= 1000					; lower side resistor in Ohms
;.equ	R2			= 2200					; upper side resistor in Ohms
.equ	Ref			= 2.56					; internal reference voltage
;.equ	VoltThreshold = LowVoltage*R1/(R1+R2)*256/Ref calculation formula, does not work
.equ	VoltThreshold = 0x88

;*****************************************************************************************
;	SRAM-Variablen
;*****************************************************************************************

.equ	VoltMean1	= 1					; first value for mean value generation
.equ	VoltMean2	= 2
.equ	VoltMean3	= 3
.equ	VoltMean4	= 4
.equ	VoltMean5	= 5
.equ	VoltMean6	= 6
.equ	VoltMean7	= 7
.equ	VoltMean8	= 8

;*****************************************************************************************
;	EEPROM-Werte
;*****************************************************************************************

.equ	PulseSwitch	= 0						; Register to store switch pulse length
.equ	VoltageThreshold = 1				; Register to store the undervoltage threshold value

;*****************************************************************************************
;	Werte
;*****************************************************************************************

;.equ	avradress=0b11000001	; avr address for receiving/transmitting

;*****************************************************************************************
;	Programmcode
;*****************************************************************************************
.cseg
.org	$0000

;*****************************************************************************************
;	Interruptroutinen
;*****************************************************************************************

		rjmp	START					; reset vector
		rjmp	INT0ROUTINE				; Int0 Pin
		reti;	PINCHANGEDROUTINE		; I/O-Pin changed
		reti;	TIM1COMPAROUTINE		; Timer/Counter1 Compare A
		rjmp	TIM1OVFROUTINE			; Timer/Counter1 Overflow
		rjmp	TIM0OVFROUTINE			; Timer/Counter0 Overflow
		reti;	EEREADYROUTINE			; EEPROM ready
		reti;	ANACOMPROUTINE			; analog comparator event
		rjmp	ADCROUTINE				; ADC Complete

;*****************************************************************************************
;	Programmstart
;*****************************************************************************************
START:
		;ldi		Temp, LOW(RAMEND)						; init stack pointer
		;out		SPL, Temp
		ldi		Temp, 0b00010010						; set PB1, PB4 output, rest input
		out		DDRB, Temp
		ldi		Temp, 0b00010000						; switch nmos and pmos off
		out		PORTB, Temp
		clr		TimeL
		clr		TimeH
		clr		AdcCount
		rcall	DELAY1s									; delay routine during rx startup
		rcall	LONG									; send one beep
		rcall	DELAY1s									; wait for the user
														; activate the fast timer to measure the pulse width
		ldi		Temp, 0b00001001						; set tmr1 prescaler to 1/16 fclk, 100kHz Timer frequency
		out		TCCR1, Temp								; CTC1, PWM1, COM1A1, COM1A0, CS13, CS12, CS11, CS10
		ldi		Temp, 0b00000100						; enable timer1 overflow
		out		TIMSK, Temp								;  ,OCIE1A,,,TOIE1,TOIE0,
		clr		Temp									; setup the beeper off time
		ldi		Status1, 0b00000010						; set setup beep off time state
		ldi		Temp, 0b00000011						; set int0 to rising edge
		out		MCUCR, Temp								;  , PUD, SE, SM1, SM0, , ISC01, ISC00
		in		Temp, GIMSK								;  , INT0, PCIE, , , , , 
		ori		Temp, 0b01000000						; enable int0
		out		GIMSK, Temp
		ldi		PulseCount, 8							; do 8 value mean
		sei												; enable interrupts
 S_OFF_WAIT:
		cpi		PulseCount, 0
		brne	S_OFF_WAIT								; measure 8 times
		;in		Temp, GIMSK
		;andi	Temp, 0b10111111						; disable int0 after routine finished
		;out		GIMSK, Temp
		cli												; disable interrupts
		lsr		TimeH									; divide by 8
		ror		TimeL
		lsr		TimeH
		ror		TimeL
		lsr		TimeH
		ror		TimeL
		mov		r0, TimeL								; store the beep off time
		clr		TimeL
		clr		TimeH
		rcall	LONG
		rcall	LONG									; beep 2 times
		rcall	DELAY1s									; wait for the user
		ldi		Status1, 0b00000100						; set setup beep on time state
		ldi		Temp, 0b00000011						; set int0 to rising edge
		out		MCUCR, Temp								;  , PUD, SE, SM1, SM0, , ISC01, ISC00
		;in		Temp, GIMSK
		;ori		Temp, 0b01000000						; enable int0
		;out		GIMSK, Temp
		sei												; enable interrupts
		ldi		PulseCount, 8							; do 8 value mean
 S_ON_WAIT:
 		cpi		PulseCount, 0
		brne	S_ON_WAIT								; measure 8 times
		;in		Temp, GIMSK
		;andi	Temp, 0b10111111						; disable int0
		;out		GIMSK, Temp
		cli												; disable interrupts
		lsr		TimeH									; divide by 8
		ror		TimeL
		lsr		TimeH
		ror		TimeL
		lsr		TimeH
		ror		TimeL
		mov		Temp3, TimeL							; backup of the Pulse length
		ldi		Temp, PulseDiff							; test if on and off pulse have same length
		add		Temp, TimeL
		sub		Temp, r0
		brlo	INIT_CALC_MEAN							; use mean value if difference is suficiently large
		cpi		Temp, 2*PulseDiff
		brsh	INIT_CALC_MEAN
		ldi		Temp, PulseSwitch
		out		EEAR, Temp								; else load the value from eeprom
		sbi		EECR, EERE								; init EEPROM read
		in		Temp, EEDR								; take the data
		cpi		Temp, 0xFF								; test if EEPROM is empty
		brne	INIT_USE_EEPROM							; no, use the value
		ldi		Temp, MidDefault						; yes, use the default value
		rcall	SHORT									; send the tone
INIT_USE_EEPROM:
		mov		TimeMid, Temp							; and use it
		cp		TimeMid, Temp3
		brlo	INIT_TEST_VOLT_LIM
INIT_USE_EEPROM_AFTER:
		rcall	SHORT									; signal
		rjmp	INIT_CALC_AF_MEAN						; go on with normal sequence
INIT_CALC_MEAN:											; calculate the mean value between min and max pulse time
		clr		Temp
		add		TimeL, r0								; add min and max time
		adc		TimeH, Temp
		lsr		TimeH									; divide by 2
		ror		TimeL
		mov		TimeMid, TimeL							; store the mean value
		ldi		Temp, PulseSwitch
		out		EEAR, Temp								; first look for the old EEPROM variable
		sbi		EECR, EERE
		in		Temp, EEDR								; read the old value
		ldi		Temp2, ThreshDiff						; use the max. difference
		add		Temp, Temp2
		cp		Temp, TimeL
		brlo	INIT_CALC_MEAN_STORE					; store new value if difference too large
		cpi		Temp, 2*ThreshDiff
		brsh	INIT_CALC_MEAN_STORE
		rjmp	INIT_CALC_AF_MEAN						; dont store if difference small
INIT_CALC_MEAN_STORE:									; store the value to eeprom
		ldi		Temp, PulseSwitch						; load EEPROM adress
		mov		Temp2, TimeMid
		out		EEAR, Temp
		out		EEDR, Temp2								; use the new value
		sbi		EECR, EEMWE
		sbi		EECR, EEWE								; write the data
		rcall	LONG									; send the stored signal
INIT_CALC_AF_MEAN:
		rcall	LONG									; beep again
		ldi		Temp, VoltageThreshold					; point to undervoltage register pointer
		out		EEAR, Temp
		sbi		EECR, EERE								; load the undervoltage value
		in		Temp, EEDR
		cpi		Temp, 0xFF								; test if empty
		brne	INIT_CALC_AF_USE						; use the value if not empty
		ldi		Temp, VoltThreshold						; if empty use factory default
INIT_CALC_AF_USE:
		mov		Undervoltage, Temp						; store the value
		ldi		Temp, 0b00000101						; set tmr0 prescaler to 1024 -- 1.6kHz Timer frequency, 1/4s overflow time
		out		TCCR0, Temp								;  , , , , , CS02, CS01, CS00
		ldi		Temp, 0b00000110						; enable timer 0 and 1 ovf int
		out		TIMSK, Temp								;  , OCIE1A, , , , TOIE1, TOIE0
		ldi		Status1, 0b00000000
		ldi		Temp, 0b00000011						; set int0 to rising edge
		out		MCUCR, Temp								;  , PUD, SE, SM1, SM0, , ISC01, ISC00
		;in		Temp, GIMSK
		;ori		Temp, 0b01000000						; enable int0
		;out		GIMSK, Temp
		rcall	DELAY1s
		ldi		Temp, 0b10100010							; setup internal reference and PB3 as AD input
		out		ADMUX, Temp								; REFS1,REFS0,ADLAR, , ,MUX2,MUX1,MUX0
		ldi		Temp, 0b10101101						; enable ADC free running, interrupt and set prescaler to 1kHz sampling rate
		out		ADCSR, Temp								; ADEN,ADSC,ADFR,ADIF,ADIE,ADPS2,ADPS1,ADPS0
		sbi		ADCSR, 6
		ldi		Temp, 0b00001111						; enable Watchdog with 1s count
		out		WDTCR, Temp								; , , ,WDTOE,WDE,WDP2,WDP1,WDP0
		sei
		andi	Status1, 0b00001111						; clear all beep states

MAIN:												; main routine
		sbrc	Status1, 4
		rcall	LOWBATT
		sbrc	Status1, 5
		rcall	SHIT
		sbrc	Status1, 6
		rcall	HELPME
		sbrc	Status1, 7
		rcall	FINDME
		rjmp	MAIN

INIT_TEST_VOLT_LIM:
		rcall	LONG									; send signal
		rcall	LONG
		rcall	LONG
		clr		Status1
		ldi		Temp, 0b00000011						; set int0 to rising edge
		out		MCUCR, Temp	
		;in		Temp, GIMSK
		;ori		Temp, 0b01000000						; enable int0
		;out		GIMSK, Temp
		sei												; enable interrupts
		rcall	DELAY1s
		andi	Status1, 0b00001001						; reset all beep states
		rcall	DELAY1s
		sbrs	Status1, 7								; go back if normal signal
		rjmp	INIT_USE_EEPROM_AFTER
		;in		Temp, GIMSK
		;andi	Temp, 0b10111111						; disable int0 after routine finished
		;out		GIMSK, Temp
		cli												; disable interrupts
		ldi		Temp, 0b10100010						; setup internal reference and PB3 as AD input
		out		ADMUX, Temp								; REFS1,REFS0,ADLAR, , ,MUX2,MUX1,MUX0
		ldi		Temp, 0b10000101						; set ADC to single conversion and wait
		out		ADCSR, Temp								; ADEN,ADSC,ADFR,ADIF,ADIE,ADPS2,ADPS1,ADPS0
		sbi		ADCSR, 6								; start one conversion
		rcall	DELAY1s									; wait
		in		Temp, ADCH								; read the ADC value
		mov		Undervoltage, Temp
		ldi		Temp2, VoltageThreshold					; point to the undervoltage storage register
		out		EEAR, Temp2
		out		EEDR, Temp								; use the new value
		sbi		EECR, EEMWE
		sbi		EECR, EEWE								; write the data
		rcall	LONG									; send the stored signal
		andi	Status1, 0b00001111						; reset all beep states
		rjmp	INIT_USE_EEPROM_AFTER

;*****************************************************************************************
;	Interruptroutinen
;***************************************************************************************** 

INT0ROUTINE:										; interrupt pin 0 pulse edge occured
		in		r9, SREG
		sbrc	Status1, 0
		rjmp	INT0ROUTINE_STOP_MEAS					; stop the measuring if bit0 is set
		clr		Temp									; start the measuring timer
		out		TCNT1, Temp
		ldi		Temp, 0b00000010
		out		MCUCR, Temp								; set interrupt to negative edge
		ori		Status1, 0b00000001						; set measure state
		rjmp	INT0ROUTINE_OUT
 INT0ROUTINE_STOP_MEAS:
		in		Temp, TCNT1								; read the measured time
		ori		Status1, 0b00001000						; set pulse occured state
		mov		Temp2, Status1							; test for the different states
		andi	Temp2, 0b00000110
		cpi		Temp2, 0b00000010
		brlo	INT0ROUTINE_NORMAL
														; setup routine
		ldi		Temp2, PulseDiffSetup					; setup difference of subsequent intervals
		add		Temp2, Temp								; pulse difference test
		sub		Temp2, r1								; compare with last pulse
		brlo	INT0ROUTINE_SETUP_JITTER
		cpi		Temp2, 2*PulseDiff						; difference is larger than 10
		brsh	INT0ROUTINE_SETUP_JITTER
		inc		Temp3									; increase the unjittered pulse counter
		cpi		Temp3, 5								; at least the last 5 pulses have to have the same value
		brlo	INT0ROUTINE_POS_OUT						; otherwise wait for the next pulse
		dec		Temp3									; prevent Temp3 from overflow
		clr		Temp2									; measure the min or max time
		add		TimeL, Temp								; add the timer values for mean value calculation
		adc		TimeH, Temp2
		dec		PulseCount								; decrease the mean value counter
		rjmp	INT0ROUTINE_POS_OUT
 INT0ROUTINE_SETUP_JITTER:								; clear unjittered pulses register and go out
		clr		Temp3
		rjmp	INT0ROUTINE_POS_OUT
 INT0ROUTINE_NORMAL:									; normal operation
		ldi		Temp2, PulseDiff						; for pulse difference test
		add		Temp2, Temp
		sub		Temp2, r1								; difference of subsequent intervals
		brlo	INT0ROUTINE_PULSE_JITTER				; difference is larger than 10
		cpi		Temp2, 2*PulseDiff						; difference is larger than 10
		brsh	INT0ROUTINE_PULSE_JITTER
		cp		Temp, TimeMid							; test if beep or not
		brsh	INT0ROUTINE_PULSE_LONG					; pulse failed
		clr		PulseCount								; set wrong pulses to zero
		;andi	Status1, 0b00001111						; clear beep state
		rjmp	INT0ROUTINE_POS_OUT						; go out
 INT0ROUTINE_PULSE_JITTER:
		inc		PulseCount								; increase Pulse fail register
		cpi		PulseCount, 8							; alarm when 8 pulses failed
		brlo	INT0ROUTINE_POS_OUT
 		dec		PulseCount								; to prevent overflow
		ori		Status1, 0b00100000						; set beep state
		rjmp	INT0ROUTINE_POS_OUT						; go out
 INT0ROUTINE_PULSE_LONG:
		inc		PulseCount								; increase Pulse fail register
		cpi		PulseCount, 8							; alarm when 8 pulses failed
		brlo	INT0ROUTINE_POS_OUT
 		dec		PulseCount								; to prevent overflow
		ori		Status1, 0b10000000						; set beep state
 INT0ROUTINE_POS_OUT:
		mov		r1, Temp								; store pulse length
 		andi	Status1, 0b11111110						; reset measure state
		ldi		Temp, 0b00000011						; set interrupt to positive edge
		out		MCUCR, Temp
 INT0ROUTINE_OUT:
		out		SREG, r9
		reti

TIM1OVFROUTINE:										; timer1 overflow pulse too long, reset
		in		r9, SREG
		sbrs	Status1, 0								; ignore if no pulse active
		rjmp	TIM1OVFROUTINE_OUT
		clr		Temp3									; clear the unjittered pulse counter
		inc		PulseCount								; increase Pulse fail register
		cpi		PulseCount, 8							; alarm when 8 pulses failed
		brlo	INT0ROUTINE_POS_OUT
 		dec		PulseCount								; to prevent overflow
		ori		Status1, 0b01000000						; set beep state
 TIM1OVFROUTINE_OUT:
		out		SREG, r9
		reti

TIM0OVFROUTINE:										; timer0 overflow this tests if during the last 1/6 s
		in		r9, SREG
		wdr												; at least one pulse occured, otherwise alarm
		sbrs	Status1, 3								; test if pulse occured
		ori		Status1, 0b01000000						; no, alarm
		andi	Status1, 0b11110111						; reset pulse occured bit
		sbrs	Status1, 3								; test if pulse occured
		clr		Temp3									; clear unjittered pulse counter
		out		SREG, r9
		reti

ADCROUTINE:
		in		r9, SREG
		in		Temp3, ADCH								; read the dac value
		;mov		Temp3, Temp
		rjmp	ADCROUTINE_NOMEAN
 ADCROUTINE_NOMEAN:
		cp		Temp3, Undervoltage						; check if undervoltage
		brsh	ADCROUTINE_NORM_VOLT
		ori		Status1, 0b00010000						; set undervoltage state if low voltage
 ADCROUTINE_NORM_VOLT:
		sbi		ADCSR, 6
		out		SREG, r9
		reti

;*****************************************************************************************
;	Subroutinen
;*****************************************************************************************

LOWBATT:											; morse sequence "low batt"
		rcall	SHORT
		rcall	SHORT
		rcall	BUCHEND
		rcall	LONG
		rcall	LONG
		rcall	LONG
		rcall	BUCHEND
		rcall	SHORT
		rcall	LONG
		rcall	LONG
		rcall	WORDEND
		rcall	LONG
		rcall	SHORT
		rcall	SHORT
		rcall	SHORT
		rcall	BUCHEND
		rcall	SHORT
		rcall	LONG
		rcall	BUCHEND
		rcall	LONG
		rcall	BUCHEND
		rcall	LONG
		rcall	WORDEND
		rcall	WORDEND
		andi	Status1, 0b00001111						; reset low batt state
		ret

SHIT:												; morse sequence "shit"
		rcall	SHORT
		rcall	SHORT
		rcall	SHORT
		rcall	BUCHEND
		rcall	SHORT
		rcall	SHORT
		rcall	SHORT
		rcall	SHORT
		rcall	BUCHEND
		rcall	SHORT
		rcall	SHORT
		rcall	BUCHEND
		rcall	LONG
		rcall	WORDEND
		rcall	WORDEND
		andi	Status1, 0b00001111
		ret

HELPME:												; morse sequence "help me"
		rcall	SHORT
		rcall	SHORT
		rcall	SHORT
		rcall	SHORT
		rcall	BUCHEND
		rcall	SHORT
		rcall	BUCHEND
		rcall	SHORT
		rcall	LONG
		rcall	SHORT
		rcall	SHORT
		rcall	BUCHEND
		rcall	SHORT
		rcall	LONG
		rcall	LONG
		rcall	SHORT
		rcall	WORDEND
		rcall	LONG
		rcall	LONG
		rcall	BUCHEND
		rcall	SHORT
		rcall	WORDEND
		rcall	WORDEND
		andi	Status1, 0b00001111
		ret

FINDME:
		rcall	SHORT
		rcall	SHORT
		rcall	LONG
		rcall	SHORT
		rcall	BUCHEND
		rcall	SHORT
		rcall	SHORT
		rcall	BUCHEND
		rcall	LONG
		rcall	SHORT
		rcall	BUCHEND
		rcall	LONG
		rcall	SHORT
		rcall	SHORT
		rcall	WORDEND
		rcall	LONG
		rcall	LONG
		rcall	BUCHEND
		rcall	SHORT
		rcall	WORDEND
		rcall	WORDEND
		andi	Status1, 0b00001111
		ret

LONG:												; one long beep signal
		cbi		PORTB, 4								; set pmos output low
		ldi		BeepTemp2, 60
 LONG_LOOP_1:
		ldi		BeepTemp, 250
  LONG_LOOP_2:
		sbi		PORTB, 1								; set nmos output high
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		cbi		PORTB, 1								; set nmos output low
		dec		BeepTemp
		brne	LONG_LOOP_2
		dec		BeepTemp2
		brne	LONG_LOOP_1
		sbi		PORTB, 4								; set pmos output high
		ldi		DelayTemp2, 131							; 1MZ
 DELAYCOUNT120lms:
		ldi		DelayTemp1, 255							; 1MZ
 DELAYCOUNTl:
 		dec		DelayTemp1								; 1MZ
		brne	DELAYCOUNTl								; 1/2MZ
 		dec		DelayTemp2								; 1MZ
		brne	DELAYCOUNT120lms						; 1/2MZ
		ret

SHORT:												; one 500ms beep signal
		cbi		PORTB, 4							; set pmos output low
		ldi		BeepTemp2, 20
 SHORT_LOOP_1:
		ldi		BeepTemp, 250
  SHORT_LOOP_2:
		sbi		PORTB, 1							; set nmos output high
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		cbi		PORTB, 1								; set nmos output low
		dec		BeepTemp
		brne	SHORT_LOOP_2
		dec		BeepTemp2
		brne	SHORT_LOOP_1
		sbi		PORTB, 4								; set pmos output high
		ldi		DelayTemp2, 131							; 1MZ
 DELAYCOUNT120sms:
		ldi		DelayTemp1, 255							; 1MZ
 DELAYCOUNTs:
 		dec		DelayTemp1								; 1MZ
		brne	DELAYCOUNTs								; 1/2MZ
 		dec		DelayTemp2								; 1MZ
		brne	DELAYCOUNT120sms						; 1/2MZ
		ret

BUCHEND:
		ldi		DelayTemp3, 2
 BECOUNTps:
		ldi		DelayTemp2, 130							; 1MZ
 BECOUNTsms:
		ldi		DelayTemp1, 255							; 1MZ
 BECOUNTs:
 		dec		DelayTemp1								; 1MZ
		brne	BECOUNTs								; 1/2MZ
 		dec		DelayTemp2								; 1MZ
		brne	BECOUNTsms								; 1/2MZ
		dec		DelayTemp3
		brne	BECOUNTps
		ret

WORDEND:
		ldi		DelayTemp3, 4
 WECOUNTps:
		ldi		DelayTemp2, 130							; 1MZ
 WECOUNTsms:
		ldi		DelayTemp1, 255							; 1MZ
 WECOUNTs:
 		dec		DelayTemp1								; 1MZ
		brne	WECOUNTs								; 1/2MZ
 		dec		DelayTemp2								; 1MZ
		brne	WECOUNTsms								; 1/2MZ
		dec		DelayTemp3
		brne	WECOUNTps
		ret

DELAY1s:											; 1s delay
		ldi		DelayTemp3, 8
 DELAYCOUNT1s:
		ldi		DelayTemp2, 255						; 1MZ
 DELAYCOUNT120mms:
		ldi		DelayTemp1, 255					; 1MZ
 DELAYCOUNT:
 		dec		DelayTemp1						; 1MZ
		brne	DELAYCOUNT						; 1/2MZ
 		dec		DelayTemp2							; 1MZ
		brne	DELAYCOUNT120mms						; 1/2MZ
		dec		DelayTemp3
		brne	DELAYCOUNT1s
		ret

